""" Configuration related functions.
"""

import configparser
import os
from pathlib import Path


home = Path.home()

user_confdir = f'{home}/.config/daily'
user_conf = f'{user_confdir}/daily.ini'


default = f"""\
[default]

# Path to journal
journal = {home}/.local/share/daily

# Store entries as md or rst
entry_format = md
"""


def get_defaults():
    """ Default values for items that should be in a config file.

    These values will be overuled by existing config entries. Useful in
    the event a config file is missing an entry.

    Returns:
        A dictionary that contains all expected entries for a
        configuration file.
    """
    return {
        'journal': f'journal = {home}/.local/share/daily',
        'entry_format': 'rst',
    }


def do_first_time_setup():
    """ Create default ini in user conf path.
    """
    try:
        os.makedirs(user_confdir)
    except FileExistsError:
        pass

    with open(user_conf, 'w') as f:
        f.write(default)

    os.makedirs(f'{home}/.local/share/daily', exist_ok=True)


def add_config_args(args, config=None):
    """ Add params from a config file to an ArgumentParser.

    Parameters are only copied if not already set in the
    ArgumentParser.

    Args:
        args: ArgumentParser instance.
        config: Path to config file.

    Returns:
        namedtuple containing config parameters overridden by command
        line arguments.

    Raises:
        FileNotFoundError: Provided config file doesn't exist.
        KeyError: Configuration file has no default section (or no sections).
    """
    config = user_conf if not config else config

    if not os.path.exists(config):
        raise FileNotFoundError('Config {} does not exist.'.format(config))

    cp = configparser.ConfigParser()

    try:
        cp.read(config)
    except Exception as e:
        raise KeyError('Config "{}" is invalid. {}.'.format(config, e))

    if 'default' not in cp:
        raise KeyError('Config {} has no "default" section.'.format(config))

    d = get_defaults()
    d.update(cp['default'])

    # copy vals into args if not already in args.
    for key, val in d.items():
        try:
            if not getattr(args, key):
                setattr(args, key, val)
        except AttributeError:
            setattr(args, key, val)

    return args



from daily_cli.parsergroups import create_filter_opts


filter_opts = create_filter_opts()


def add_subparser(subparsers):
    sp = subparsers.add_parser(
        'todo',
        help='Display todos.',
        description=(
            'These are any entries with an "events" heading within the'
            '(default) range of 2 weeks.'),
        parents=[filter_opts])



""" daily argument parser.
"""

import argparse
import sys

from daily_cli import __version__
from daily_cli.cli import build_out_subparsers
from daily_cli.parsergroups import create_filter_opts


filter_opts = create_filter_opts()


def print_version():
    class printVersion(argparse.Action):
        def __call__(self, parser, args, values, option_string=None):
            print(__version__)
            sys.exit(0)
    return printVersion


def create_parser():
    """ Create main parser.

    Returns:
        Reference to the parser. Parse main command line args with
            parser.parse_args().
    """
    parser = argparse.ArgumentParser(
        prog='daily',
        description=(
            'The command-line journal for daily entries. Run "daily" with no '
            'arguments to perform first-time setup.'))

    parser.add_argument(
        '--config',
        help='Select custom configuration file.')

    parser.add_argument(
        '-j', '--journal',
        help='Specify the journal to operate on.')

    parser.add_argument(
        '-f', '--entry-format', choices=['md', 'rst'],
        help='Text format of the entries.')

    parser.add_argument(
        '--version', nargs=0, help='Print the version of daily and exit.',
        action=print_version())

    # begin subparsers
    subparsers = parser.add_subparsers(
        metavar='command',
        dest='command',
        description='Each has its own [-h, --help] statement.')

    build_out_subparsers(subparsers)

    return parser



""" main entry point for daily.
"""

import importlib
import os
import sys

import argcomplete

from daily_cli.parser import create_parser
from daily_cli.config import add_config_args, do_first_time_setup, user_conf


def main():
    parser = create_parser()
    argcomplete.autocomplete(parser)
    args = parser.parse_args()

    if not os.path.exists(user_conf) and not args.config:
        print('INFO: Running first time setup')
        do_first_time_setup()
        if not args.command:
            print('Setup complete. User config created in {}'.format(
                user_conf))
            print('Run "daily -h" to see usage help.')
            sys.exit(0)
    elif not args.config:
        args.config = user_conf

    if not args.command:
        parser.print_help()
        sys.exit(1)

    # fill in args with values from config.
    args = add_config_args(args, args.config)

    subcommand = importlib.import_module('daily_cli.cli.{}.main'.format(args.command))
    subcommand = getattr(subcommand, 'main'.format(args.command))

    try:
        subcommand(args)
    except KeyboardInterrupt:
        print('Interrupt caught - closing.')

    sys.exit(0)


if __name__ == '__main__':
    main()



""" todo subcommand.

This command is basically just "show", but with some preset filters.
It shows todos between 2 weeks from now.
"""
from libzet import zettels_to_str

from daily_todo.load import get_title_from_date, load_entries


def main(args):

    # default date range of +2 weeks
    if not args.before and not args.after:
        args.after = get_title_from_date('today')
        args.before = get_title_from_date('2 weeks from now')

    try:
        entries = load_entries(args)
    except FileNotFoundError as e:
        print(f'ERROR: "{args.journal}" does not exist.')
        sys.exit(1)
    except PermissionError as e:
        print(f'ERROR: "{args.journal}" could not be read.')
        sys.exit(1)
    except ValueError as e:
        print(f'ERROR: "{args.journal}" contains invalid entries. {e}')
        sys.exit(1)

    print(zettels_to_str(entries, args.entry_format, headings=['todo']))



import sys

from libzet import zettels_to_str

from daily_cli.load import get_title_from_date, load_entries


def main(args):
    if args.entry_format not in ['rst', 'md']:
        print(f'ERROR: The entry_format setting needs to be either rst or md.')
        sys.exit(1)

    if not any([args.after, args.before, args.date, args.tags]):
        args.after = get_title_from_date('2 weeks ago')
        args.before = get_title_from_date('today')

    try:
        entries = load_entries(args)
    except FileNotFoundError as e:
        print(f'ERROR: "{args.journal}" does not exist.')
        sys.exit(1)
    except PermissionError as e:
        print(f'ERROR: "{args.journal}" could not be read.')
        sys.exit(1)
    except ValueError as e:
        print(f'ERROR: "{args.journal}" contains invalid entries. {e}')
        sys.exit(1)

    print(zettels_to_str(entries, args.entry_format, args.headings))



import os
import sys

from libzet import create_zettel, edit_zettels, load_zettels, load_zettels, move_zettels, save_zettels
from superdate import parse_date

from daily_cli.load import get_title_from_date, load_entries


def _get_filepath(date_, journal, format_):
    """ What should the path to an entry be?

    Args:
        date_: Date of entry
        journal: Directory containing entries.
        format: file extension of entry

    Returns:
        Expected filepath.
    """
    exp = parse_date(date_).strftime(f'%Y-%m-%d.{format_}')
    return os.path.sep.join([journal, exp])


def _correct_entry(z, args):
    """ Correct and save an entry.

    Use like z = _correct_entry(z, entry_format)

    Returns:
        A ref to the corrected entry.
    """
    if 'tags' not in z.attrs:
        z.attrs['tags'] = []

    z.attrs['tags'] = sorted(list(set(z.attrs['tags'])))

    # Sync date and filepath
    z.title = get_title_from_date(z.title)

    exp_path = _get_filepath(z.title, args.journal, args.entry_format)
    if '_loadpath' not in z.attrs:
        z.attrs['_loadpath'] = exp_path

    if z.attrs['_loadpath'] != exp_path:
        z = move_zettels(z, exp_path)[0]
    else:
        save_zettels(z, args.entry_format)

    return z


def main(args):
    """ Add a new entry or update existing entries.
    """
    editor = os.environ.get('EDITOR') or os.environ.get('VISUAL')
    if not editor:
        print(
            'ERROR: No EDITOR or VISUAL environment variable defined. '
            'Export one of those for your preferred text editor and re-run.')
        sys.exit(1)

    dir_ = os.path.dirname(args.journal)
    if os.path.isfile(dir_):
        print('ERROR: {} not a directory.'.format(dir_))
        sys.exit(1)

    # Possibly create and save a new entry for the date.
    if not any([args.date, args.before, args.after]):
        args.date = 'today'

    if args.date:
        new_zet = _get_filepath(args.date, args.journal, args.entry_format)
        try:
            z = create_zettel(new_zet, title=args.date, zettel_format=args.entry_format, no_edit=True)
        except FileExistsError:
            z = load_zettels(new_zet, args.entry_format)[0]

        _correct_entry(z, args)

    # Load entries
    try:
        entries = load_entries(args)
    except ValueError as e:
        print(f'ERROR: {e}')
        sys.exit(1)

    if not entries:
        print('No matching entries found.')
        sys.exit(1)

    # Edit and save
    modified = edit_zettels(entries, args.entry_format, args.headings, f'failed-adds.{args.entry_format}', delete=True)
    [_correct_entry(z, args) for z in modified]
